@@ -21,7 +21,15 @@ class Wrapper(object):
     def _deferToThread(self, f, *a):
         """
         Defers to the thread pool.
+
+        If the thread pool has not been started yet, starts the thread
+        pool and schedules it to be stopped before the reactor stops.
         """
+        if not self.threadPool.started:
+            self.threadPool.start()
+            self.reactor.addSystemEventTrigger(
+                "before", "shutdown", self.threadPool.stop)
+
         return threads.deferToThreadPool(self.reactor, self.threadPool, f, *a)
 
 
